/*  Main pn
    Elektroniklabor Knallinger
    Copyright  1993. Alle Rechte vorbehalten.

    SYSTEM:       pn.exe Application
    DATEI:        APXPrint.CPP
    AUTOR:        Wolfgang Knallinger


    UEBERBLICK
    ==========
    Quelltextdatei fuer die Implementation von Druck-Operationen
*/


#include <owl\owlpch.h>
#pragma hdrstop

#include <owl\listbox.h>
#include <owl\edit.h>
#include <owl\preview.h>

#include "apxprint.h"


// Druckbereich im Druck-Dialog nicht aktivieren, da nur eine Seite
// verfuegbar ist
//
void APXPrintOut::GetDialogInfo (int& minPage, int& maxPage, int& selFromPage, int& selToPage)
{
    minPage = maxPage = 0;
    selFromPage = selToPage = 0;
}


void APXPrintOut::BeginPrinting ()
{
    TRect clientR;

    BeginPage(clientR);

    TFrameWindow *fWindow = TYPESAFE_DOWNCAST(Window, TFrameWindow);

    HFONT   hFont = (HFONT)fWindow->GetClientWindow()->GetWindowFont();
    TFont   font("Arial", -12);
    if (hFont == 0)
      DC->SelectObject(font);
    else
      DC->SelectObject(TFont(hFont));
    
    TEXTMETRIC  tm;
    int fHeight = (DC->GetTextMetrics(tm) == TRUE) ? tm.tmHeight + tm.tmExternalLeading : 10;

    DC->RestoreFont();
    
    // Wieviele Zeilen dieser Schrift passen auf eine Seite
    int linesPerPage = MulDiv(clientR.Height(), 1, fHeight);
    
    TPrintDialog::TData &printerData = Printer->GetSetup();

    int maxPg = 1;

    // Client-Fenster ermitteln
    TEdit *clientEditWindow = 0;
    TListBox *clientListWindow = 0;

    clientEditWindow = TYPESAFE_DOWNCAST(fWindow->GetClientWindow(), TEdit);
    if (clientEditWindow)
        maxPg = ((clientEditWindow->GetNumLines() / linesPerPage) + 1.0);
    else {
        clientListWindow = TYPESAFE_DOWNCAST(fWindow->GetClientWindow(), TListBox);
        if (clientListWindow)
            maxPg = ((clientListWindow->GetCount() / linesPerPage) + 1.0);
    }

    // Anzahl der zu druckenden Seiten ermitteln
    printerData.MinPage = 1;
    printerData.MaxPage = maxPg;

    EndPage();

    TPrintout::BeginPrinting();
}


void APXPrintOut::BeginPage (TRect &clientR)
{
    TScreenDC screenDC;
    TSize screenRes(screenDC.GetDeviceCaps(LOGPIXELSX),
                    screenDC.GetDeviceCaps(LOGPIXELSY));
    TSize printRes(DC->GetDeviceCaps(LOGPIXELSX),
                   DC->GetDeviceCaps(LOGPIXELSY));

    // Die Fenstergroesse wird voruebergehend geaendert. Beachten Sie, dass
    // hier nicht AdjustWindowRect verwendet wird, da es nicht genau arbeitet.
    // Stattdessen wird der Unterschied (in Pixel) zwischen dem Client-Fenster und
    // dem Rahmenfenster berechnet und zu clientRect addiert. Das Ergebnis ist
    // die neue Groesse des Rahmenfensters fuer SetWindowPos.
    clientR = Window->GetClientRect();
    Window->MapWindowPoints(HWND_DESKTOP, (TPoint*)&clientR, 2);

    // X- und Y-Pixel berechnen, um das Client-Fenster dem Rahmenfenster anzupassen
    OrgR = Window->GetWindowRect();
    int adjX = OrgR.Width() - clientR.Width();
    int adjY = OrgR.Height() - clientR.Height();
    
    // DC auf das Fenster skalieren, damit der Ausdruck dem Fenster entspricht
    if (Scale) {
        clientR = Window->GetClientRect();
        PrevMode = DC->SetMapMode(MapMode);
        DC->SetViewportExt(PageSize, &OldVExt);

        // Fenster auf logische Seitengroesse skalieren (setzt left = top = 0 voraus)
        clientR.right = MulDiv(PageSize.cx, screenRes.cx, printRes.cx);
        clientR.bottom = MulDiv(PageSize.cy, screenRes.cy, printRes.cy);

        DC->SetWindowExt(clientR.Size(), &OldWExt);
    }

    // Neue Fenstergroesse auf Basis der DC-Dimensionen des Druckers berechnen
    Window->SetRedraw(FALSE);
    Window->SetWindowPos(0, 0, 0, clientR.Width() + adjX, clientR.Height() + adjY,
                         SWP_NOMOVE | SWP_NOREDRAW | SWP_NOZORDER| SWP_NOACTIVATE);
}


void APXPrintOut::PrintPage (int page, TRect& bandRect, unsigned)
{
    TRect clientR;

    BeginPage(clientR);

    if (Scale)
        DC->DPtoLP(bandRect, 2);

    // Druckbereich auf diese aktuelle Seite setzen
    TPrintDialog::TData& printerData = Printer->GetSetup();
    int fromPg = printerData.FromPage;
    int toPg = printerData.ToPage;

    printerData.FromPage = page;
    printerData.ToPage = page;

    // Fenster veranlassen, sich selbst auf dem Drucker-DC auszugeben
    Window->Paint(*DC, FALSE, bandRect);

    printerData.FromPage = fromPg;
    printerData.ToPage = toPg;

    if (Scale)
        DC->LPtoDP(bandRect, 2);

    EndPage();
}


void APXPrintOut::EndPage ()
{
    // Urspruengliche Fenstergroesse wiederherstellen
    Window->SetWindowPos(0, 0, 0, OrgR.Width(), OrgR.Height(),
                         SWP_NOMOVE | SWP_NOREDRAW | SWP_NOZORDER| SWP_NOACTIVATE);
    Window->SetRedraw(TRUE);

    // DC-Aenderungen rueckgaengig machen
    if (Scale) {
        DC->SetWindowExt(OldWExt);
        DC->SetViewportExt(OldVExt);
        DC->SetMapMode(PrevMode);
    }
}


BOOL APXPrintOut::HasPage (int pageNumber)
{
    TPrintDialog::TData &printerData = Printer->GetSetup();

    return (pageNumber >= printerData.MinPage) && (pageNumber <= printerData.MaxPage);
}
